@@ -30,6 +30,10 @@ def _get_context_value():
     return context_value.get()
 
 
+def _get_from_context(key, default):
+    return getattr(_get_context_value(), key, default)
+
+
 class FalsyList(list):
     def __bool__(self):
         # for Python 3
@@ -258,6 +262,48 @@ class CallbackContext:
         else:
             ctx_value.updated_props[_id] = props
 
+    @property
+    @has_context
+    def cookies(self):
+        """
+        Get the cookies for the current callback.
+        Works with background callbacks.
+        """
+        return _get_from_context("cookies", {})
+
+    @property
+    @has_context
+    def headers(self):
+        """
+        Get the original headers for the current callback.
+        Works with background callbacks.
+        """
+        return _get_from_context("headers", {})
+
+    @property
+    @has_context
+    def path(self):
+        """
+        Path of the callback request with the query parameters.
+        """
+        return _get_from_context("path", "")
+
+    @property
+    @has_context
+    def remote(self):
+        """
+        Remote addr of the callback request.
+        """
+        return _get_from_context("remote", "")
+
+    @property
+    @has_context
+    def origin(self):
+        """
+        Origin of the callback request.
+        """
+        return _get_from_context("origin", "")
+
 
 callback_context = CallbackContext()
 
